@@ -18,6 +18,8 @@ int emperor_throttle_level;
 
 struct uwsgi_instance *ui;
 
+time_t on_royal_death = 0;
+
 /*
 
 	blacklist subsystem
@@ -482,6 +484,10 @@ void uwsgi_register_imperial_monitor(char *name, void (*init) (struct uwsgi_empe
 // the sad death of an Emperor
 static void royal_death(int signum) {
 
+	if (on_royal_death) {
+		uwsgi_log("[emperor] *** RAGNAROK ALREADY EVOKED (mercyless in %d seconds)***\n", uwsgi.reload_mercy - (uwsgi_now() - on_royal_death));
+		return;
+	}
 
 	struct uwsgi_instance *c_ui = ui->ui_next;
 
@@ -502,8 +508,14 @@ static void royal_death(int signum) {
 	}
 
 	uwsgi_log("[emperor] *** RAGNAROK EVOKED ***\n");
-	uwsgi_notify("The Emperor is buried.");
-	exit(0);
+
+	while (c_ui) {
+                emperor_stop(c_ui);
+                c_ui = c_ui->ui_next;
+        }
+
+	if (!uwsgi.reload_mercy) uwsgi.reload_mercy = 30;
+	on_royal_death = uwsgi_now();
 }
 
 // massive reload of vassals
@@ -1520,8 +1532,43 @@ void emperor_loop() {
 
 	int freq = 0;
 
+	uwsgi_hooks_run(uwsgi.hook_emperor_start, "emperor-start", 1);
+
+	// signal parent-Emperor about my loyalty
+        if (uwsgi.has_emperor && !uwsgi.loyal) {
+                uwsgi_log("announcing my loyalty to the Emperor...\n");
+                char byte = 17;
+                if (write(uwsgi.emperor_fd, &byte, 1) != 1) {
+                        uwsgi_error("write()");
+                }
+                uwsgi.loyal = 1;
+        }
+
 	for (;;) {
 
+		if (on_royal_death) {
+			if (!ui->ui_next) break;
+			if (uwsgi_now() - on_royal_death >= uwsgi.reload_mercy) {
+				ui_current = ui->ui_next;
+				while (ui_current) {
+					uwsgi_log_verbose("[emperor] NO MERCY for vassal %s !!!\n", ui_current->name);
+					kill(ui_current->pid, SIGKILL);
+					ui_current = ui_current->ui_next;
+				}
+				break;
+			}
+			ui_current = ui->ui_next;
+			while (ui_current) {
+				struct uwsgi_instance *dead_vassal = ui_current;
+				ui_current = ui_current->ui_next;	
+                        	pid_t dead_pid = waitpid(dead_vassal->pid, &waitpid_status, WNOHANG);
+				if (dead_pid > 0 || dead_pid < 0) {
+					emperor_del(dead_vassal);	
+				}
+			}
+			sleep(1);
+			continue;
+                }
 
 		if (!i_am_alone) {
 			diedpid = waitpid(uwsgi.emperor_pid, &waitpid_status, WNOHANG);
@@ -1612,6 +1659,9 @@ void emperor_loop() {
 		ui_current = ui->ui_next;
 		while (ui_current) {
 			if (ui_current->last_heartbeat > 0) {
+#ifdef UWSGI_DEBUG
+				uwsgi_log("%d %d %d %d\n", ui_current->last_heartbeat, uwsgi.emperor_heartbeat,  ui_current->last_heartbeat + uwsgi.emperor_heartbeat, uwsgi_now());
+#endif
 				if ((ui_current->last_heartbeat + uwsgi.emperor_heartbeat) < uwsgi_now()) {
 					uwsgi_log("[emperor] vassal %s sent no heartbeat in last %d seconds, brutally respawning it...\n", ui_current->name, uwsgi.emperor_heartbeat);
 					// set last_heartbeat to 0 avoiding races
@@ -1705,6 +1755,10 @@ void emperor_loop() {
 
 	}
 
+	uwsgi_log_verbose("The Emperor is buried.\n");
+	uwsgi_notify("The Emperor is buried.");
+        exit(0);
+
 }
 
 void emperor_send_stats(int fd) {
@@ -1995,7 +2049,6 @@ next:
 		uwsgi.has_emperor = 1;
 		uwsgi.emperor_fd = atoi(emperor_env);
 		uwsgi.master_process = 1;
-		uwsgi.no_orphans = 1;
 		uwsgi_log("*** has_emperor mode detected (fd: %d) ***\n", uwsgi.emperor_fd);
 
 		if (getenv("UWSGI_EMPEROR_FD_CONFIG")) {
