@@ -123,7 +123,7 @@ void uwsgi_init_default() {
 	// a workers hould be running for at least 10 seconds
 	uwsgi.min_worker_lifetime = 10;
 
-	uwsgi.shared->spooler_frequency = 30;
+	uwsgi.spooler_frequency = 30;
 
 	uwsgi.shared->spooler_signal_pipe[0] = -1;
 	uwsgi.shared->spooler_signal_pipe[1] = -1;
@@ -183,6 +183,8 @@ void uwsgi_init_default() {
 
 	uwsgi.master_fifo_fd = -1;
 	uwsgi_master_fifo_prepare();
+
+	uwsgi.notify_socket_fd = -1;
 }
 
 void uwsgi_setup_reload() {
