@@ -517,12 +517,28 @@ fallback:
 			goto end;
 		}
 
+		// is it a potential virtual file (/proc, /sys...) ?
+		int is_virtual = 0;
+		if (sb.st_size == 0) {
+			is_virtual = 1;
+			sb.st_size = 4096;
+		}
+
 		buffer = uwsgi_malloc(sb.st_size + add_zero);
 
 		len = read(fd, buffer, sb.st_size);
-		if (len != sb.st_size) {
-			uwsgi_error("read()");
-			exit(1);
+		if (!is_virtual) {
+			if (len != sb.st_size) {
+			}
+		}
+		else {
+			if (len >= 0) {
+				sb.st_size = len;
+			}
+			else {
+				uwsgi_error("read()");
+				exit(1);
+			}
 		}
 
 		close(fd);
@@ -951,6 +967,7 @@ int uwsgi_read_nb(int fd, char *buf, size_t remains, int timeout) {
 ssize_t uwsgi_read_true_nb(int fd, char *buf, size_t len, int timeout) {
         int ret;
 
+	errno = 0;
 	ssize_t rlen = read(fd, buf, len);
         if (rlen > 0) {
 		return rlen;	
@@ -961,8 +978,10 @@ ssize_t uwsgi_read_true_nb(int fd, char *buf, size_t len, int timeout) {
         }
         return -1;
 wait:
+	errno = 0;
         ret = uwsgi.wait_read_hook(fd, timeout);
         if (ret > 0) {
+		errno = 0;
         	rlen = read(fd, buf, len);
                 if (rlen > 0) {
 			return rlen;
