@@ -242,6 +242,17 @@ static void master_check_listen_queue() {
 		if (uwsgi_sock->queue > 0 && uwsgi_sock->queue >= uwsgi_sock->max_queue) {
 			uwsgi_log_verbose("*** uWSGI listen queue of socket \"%s\" (fd: %d) full !!! (%llu/%llu) ***\n", uwsgi_sock->name, uwsgi_sock->fd, (unsigned long long) uwsgi_sock->queue, (unsigned long long) uwsgi_sock->max_queue);
 		}
+
+		if (uwsgi.alarm_backlog) {
+			char buf[1024];
+			int ret = snprintf(buf, 1024, "listen queue of socket \"%s\" (fd: %d) full !!! (%llu/%llu)", uwsgi_sock->name, uwsgi_sock->fd, (unsigned long long) uwsgi_sock->queue, (unsigned long long) uwsgi_sock->max_queue);
+			if (ret > 0 && ret < 1024) {
+				struct uwsgi_string_list *usl = NULL;
+				uwsgi_foreach(usl, uwsgi.alarm_backlog) {
+					uwsgi_alarm_trigger(usl->value, buf, ret);
+				}
+			}
+		}
 		uwsgi_sock = uwsgi_sock->next;
 	}
 
@@ -323,6 +334,12 @@ int master_loop(char **argv, char **environ) {
 		event_queue_add_fd_read(uwsgi.master_queue, uwsgi.master_fifo_fd);
 	}
 
+	if (uwsgi.notify_socket) {
+		uwsgi.notify_socket_fd = bind_to_unix_dgram(uwsgi.notify_socket);
+		uwsgi_log("notification socket enabled on %s (fd: %d)\n", uwsgi.notify_socket, uwsgi.notify_socket_fd);
+		event_queue_add_fd_read(uwsgi.master_queue, uwsgi.notify_socket_fd);
+	}
+
 	if (uwsgi.spoolers) {
 		event_queue_add_fd_read(uwsgi.master_queue, uwsgi.shared->spooler_signal_pipe[0]);
 	}
@@ -508,6 +525,16 @@ int master_loop(char **argv, char **environ) {
 		}
 		ud = ud->next;
 	}
+	// hook touches
+	uwsgi_foreach(usl, uwsgi.hook_touch) {
+		char *space = strchr(usl->value, ' ');
+		if (space) {
+                        *space = 0;
+                        usl->len = strlen(usl->value);
+			uwsgi_string_new_list((struct uwsgi_string_list **)&usl->custom_ptr, space+1);
+                }
+	}
+	uwsgi_check_touches(uwsgi.hook_touch);
 
 	// fsmon
 	uwsgi_fsmon_setup();
@@ -555,6 +582,7 @@ int master_loop(char **argv, char **environ) {
 	}
 
 	// here really starts the master loop
+	uwsgi_hooks_run(uwsgi.hook_master_start, "master-start", 1);
 
 	for (;;) {
 		//uwsgi_log("uwsgi.ready_to_reload %d %d\n", uwsgi.ready_to_reload, uwsgi.numproc);
@@ -807,7 +835,7 @@ int master_loop(char **argv, char **environ) {
                         			touched = uwsgi_check_touches(ud->touch);
 						if (touched) {
 							uwsgi_log_verbose("*** %s has been touched... reloading daemon \"%s\" (pid: %d) !!! ***\n", touched, ud->command, (int) ud->pid);
-							if (kill(ud->pid, ud->stop_signal)) {
+							if (kill(-ud->pid, ud->stop_signal)) {
 								uwsgi_error("[uwsgi-daemon/touch] kill()");
 							}
 						}
@@ -815,6 +843,12 @@ int master_loop(char **argv, char **environ) {
 					ud = ud->next;
                 		}
 
+				// hook touches
+				touched = uwsgi_check_touches(uwsgi.hook_touch);
+				if (touched) {
+					uwsgi_hooks_run((struct uwsgi_string_list *) touched, "touch", 0);
+				}
+
 			}
 
 			// allows the KILL signal to be delivered;
