@@ -14,6 +14,10 @@
 #define UWSGI_PYTHON_OLD
 #endif
 
+#if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION < 7
+#define HAS_NOT_PyMemoryView_FromBuffer
+#endif
+
 #if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION < 7
 #define HAS_NOT_PyFrame_GetLineNumber 
 #endif
@@ -30,6 +34,12 @@
 #define PYTHREE
 #endif
 
+#if (PY_VERSION_HEX < 0x02060000)
+#ifndef Py_SIZE
+#define Py_SIZE(ob)             (((PyVarObject*)(ob))->ob_size)
+#endif
+#endif
+
 #define UWSGI_GET_GIL up.gil_get();
 #define UWSGI_RELEASE_GIL up.gil_release();
 
@@ -185,6 +195,8 @@ struct uwsgi_python {
 
 	char *raw;
 	PyObject *raw_callable;
+
+	struct uwsgi_string_list *sharedarea;
 };
 
 
